Public Class Game
    Inherits Microsoft.Xna.Framework.Game

    Protected _spriteBatch As SpriteBatch
    Protected _vbContentManager As VBContentManager
    Protected _graphicsDeviceManager As GraphicsDeviceManager

    Private Event PrepareDeviceSettings(ByVal sender As Object, ByVal e As PreparingDeviceSettingsEventArgs)
    Protected Event InitializationComplete()

    ''' <summary>
    ''' Vrt instanci objektu ContentManager
    ''' </summary>
    Public Shadows ReadOnly Property Content() As VBContentManager
        Get
            Return _vbContentManager
        End Get
    End Property

    ''' <summary>
    ''' Vrt instanci objektu SpriteBatch
    ''' </summary>
    Public ReadOnly Property SpriteBatch() As SpriteBatch
        Get
            Return _spriteBatch
        End Get
    End Property

    Private land As Landscape       ' n tern

    Private posX As Single = 26, posZ As Single = 18, angle As Single = 0

    Private world As Matrix = Matrix.Identity
    Private view As Matrix
    Private proj As Matrix = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4, 4 / 3, 1, 1000)

    Private unicorn As Model

    Public Sub New()
        MyBase.New()

        ' Vytvoit GraphicsDeviceManager a VBContentManager
        _graphicsDeviceManager = New GraphicsDeviceManager(Me)
        _vbContentManager = New VBContentManager(Me.Services, "Content")
        AddHandler _graphicsDeviceManager.PreparingDeviceSettings, AddressOf PrepDeviceSettings

        ' >> Zde nsleduje nastaven okna hry

        With _graphicsDeviceManager
            .IsFullScreen = False                       ' fullscreen
            .SynchronizeWithVerticalRetrace = True      ' limitovat maximln poet FPS frekvenc monitoru
            .PreferredBackBufferWidth = 800             ' velikost okna 800 x 600
            .PreferredBackBufferHeight = 600
        End With

        With Me
            .Window.Title = "Moje VB.NET XNA Game"      ' titulek okna
            .Window.AllowUserResizing = False           ' zakzat uivateli mnit velikost okna
            .IsMouseVisible = True                      ' systmov kurzor myi je vidt, pokud je v okn
        End With

    End Sub

    Protected Sub PrepDeviceSettings(ByVal sender As Object, ByVal e As PreparingDeviceSettingsEventArgs)
        RaiseEvent PrepareDeviceSettings(sender, e)
    End Sub

    Protected Overrides Sub LoadContent()
        ' Zkompilovat a nast vechen hern obsah
        _vbContentManager.CompileContent(True)
        _vbContentManager.LoadAllContent()

        ' >> Zde nsleduje kd pro naten hernho obsahu
        land = New Landscape(Me.GraphicsDevice)
        land.TextureScale = 16.0F
        land.LoadTerrain(Content.Load(Of Texture2D)("Content\heightmap"), 25)
        land.Texture = Content.Load(Of Texture2D)("Content\grass")

        unicorn = Content.Load(Of Model)("Content\unicorn")
    End Sub

    Protected Overrides Sub Initialize()
        MyBase.Initialize()

        ' >> Zde nsleduje kd pro inicializaci scny

        _spriteBatch = New SpriteBatch(Me.GraphicsDevice)




        ' Vyvolat udlost dokonen inicializace
        RaiseEvent InitializationComplete()
    End Sub

    Protected Overrides Sub Update(ByVal gameTime As Microsoft.Xna.Framework.GameTime)

        ' >> Zde nsleduje kd pro pepotn scny

        Dim hopping As Double = 0
        'reagovat na ovldn klvesnic
        With Keyboard.GetState()
            If .IsKeyDown(Input.Keys.Up) Then   'jedeme dopedu
                posX += -Math.Sin(angle) * gameTime.ElapsedGameTime.Milliseconds * 0.01F        'posunout jednoroce
                posZ += Math.Cos(angle) * gameTime.ElapsedGameTime.Milliseconds * 0.01F
                hopping = 0.09 * Math.Sin(gameTime.TotalGameTime.TotalMilliseconds * 0.006F)    'efekt hopsn
            ElseIf .IsKeyDown(Input.Keys.Down) Then  'couvme
                posX -= -Math.Sin(angle) * gameTime.ElapsedGameTime.Milliseconds * 0.005F        'posunout jednoroce
                posZ -= Math.Cos(angle) * gameTime.ElapsedGameTime.Milliseconds * 0.005F
            End If
        End With

        'zatet pomoc myi
        With Mouse.GetState()
            'otet pohled
            angle += (.X - Me.Window.ClientBounds.Width / 2) * 0.006F
            'vrtit my doprosted okna
            Mouse.SetPosition(Me.Window.ClientBounds.Width / 2, Me.Window.ClientBounds.Height / 2)
        End With


        'kamera se bude dvat na msto, kde je jednoroec, ale na msto 3 jednotky nad zem
        Dim lookAt As Vector3 = New Vector3(posX, land.GetHeight(posX, posZ) + 2, posZ)
        'spotat pozici kamery v zvislosti na natoen jednoroce
        Dim camPos As Vector3 = New Vector3(1.5 * Math.Sin(angle), 0.1 + hopping, -1.5 * Math.Cos(angle))
        'vytvoit matici vhledu
        view = Matrix.CreateLookAt(lookAt + camPos, lookAt, Vector3.Up)

        MyBase.Update(gameTime)
    End Sub

    Protected Overrides Sub Draw(ByVal gameTime As Microsoft.Xna.Framework.GameTime)
        ' Smazat obrazovku
        Me.GraphicsDevice.Clear(Color.CornflowerBlue)

        ' >> Zde nsleduje kd pro vykreslen scny

        land.Draw(world, view, proj)


        RenderModel(unicorn, New Vector3(posX, land.GetHeight(posX, posZ), posZ), angle - MathHelper.PiOver2, 3.0F)


        ' Vykreslit vsledek na obrazovku
        MyBase.Draw(gameTime)
    End Sub

    Public Sub RenderModel(ByVal m As Model, ByVal position As Vector3, ByVal rotation As Double, ByVal scale As Single)
        ' Vykreslit cel model
        Dim transforms(m.Bones.Count - 1) As Matrix
        m.CopyAbsoluteBoneTransformsTo(transforms)

        For Each mesh As ModelMesh In m.Meshes      ' projt vechny sti modelu

            For Each eff As BasicEffect In mesh.Effects     ' vem efektm nastavit pozice
                ' nastavit vechny 3 matice
                eff.World = Matrix.CreateScale(scale) * Matrix.CreateRotationY(rotation) * transforms(mesh.ParentBone.Index) * Matrix.CreateTranslation(position)
                eff.View = view
                eff.Projection = proj

                eff.EnableDefaultLighting()     ' vypnout standardn osvtlen
            Next

            mesh.Draw()         ' vykreslit st modelu

        Next
    End Sub


End Class
